##################################
### ieXbeta.tcl                ###
### Version 1.0                ###
### By Wcc                     ###
### wcc@techmonkeys.org        ###
### http://www.dawgtcl.com:81/ ###
### EFnet #|DAWG|Tcl           ###
##################################

############################################################################
### Copyright  2000 - 2002 |DAWG| Scripting Group. All rights reserved. ###
############################################################################

###############################################################################
## This script retrieves news updates from http://www.ieXbeta.com and sends  ##
## them to all +ieXbeta channels. Public and dcc triggers are also provided. ##
## Please note that this script and |DAWG| Tcl are in no way affiliated with ##
## www.ieXbeta.com.                                                          ##
###############################################################################

##############
## COMMANDS ##
#################################################################
## DCC ## .ieXnews [limit] (Can be changed.)                   ##
######### Shows the last X news entries from the ieXbeta news  ##
######### page. X is defined by the default setting, below, or ##
######### by the optional "limit" (max news entries to show)   ##
######### argument.                                            ##
######### ---------------------------------------------------- ##
######### .chanset <channel> +/-ieXbeta                        ##
######### Setting a channel +ieXbeta news makes new news       ##
######### entries get announced to the channel.                ##
#################################################################
## PUB ## !ieXnews [limit] (Can be changed.)                   ##
######### Shows the last X news entries from the ieXbeta news  ##
######### page. X is defined by the default setting, below, or ##
######### by the optional "limit" (max news entries to show)   ##
######### argument.                                            ##
#################################################################

##########################################################
## Just load the script, edit the settings, and rehash. ##
##########################################################

#############################
# Set the dcc command here. #
#############################

set iexbeta_setting(dcmd) "ieXnews"

#############################
# Set the pub command here. #
#############################

set iexbeta_setting(pcmd) "!ieXnews"

###############################################################
# Set the flag required to use the pub and dcc commands here. #
###############################################################

set iexbeta_setting(flag) "-"

#########################################################
# Set the number of recent news entries to return here. #
#########################################################

set iexbeta_setting(rmax) "1"

#################################################
# Set the time between updates in minutes here. #
#################################################

set iexbeta_setting(updatetime) "10"

############################
# Set the update url here. #
############################

set iexbeta_setting(url) "www.iexbeta.com/index.php"

###############################
# Set the database file here. #
###############################

set iexbeta_setting(database) "./scripts/iexbeta.db"

###################################
# Enable use of bold in DCC chat? #
###################################

set iexbeta_setting(bold) 1

#########################################
# Set the reply mode here:              #
#   0 - Private message to the channel. #
#   1 - Notice to the channel.          #
#   2 - Private message to the nick.    #
#   3 - Notice to the nick.             #
#########################################

set iexbeta_setting(rplmode) "0"

###########################################
# Prefix "IEXBETA:" in DCC chat messages? #
###########################################

set iexbeta_setting(IEXBETA:) 1

#############################
# Enable bold in headlines? #
#############################

set iexbeta_setting(hbold) 1

####################
# Code begins here #
####################

if {![string match 1.6.* $version]} { putlog "\002IEXBETA:\002 \002WARNING:\002 This script is intended to run on eggdrop 1.6.x." }
if {[info tclversion] < 8.2} { putlog "\002IEXBETA:\002 \002WARNING:\002 This script is intended to run on Tcl Version 8.2 or later." }

bind dcc $iexbeta_setting(flag) $iexbeta_setting(dcmd) iexbeta_dcc
bind pub $iexbeta_setting(flag) $iexbeta_setting(pcmd) iexbeta_pub

package require http
setudef flag iexbeta

proc iexbeta_readarray {array file} {
	upvar $array ours
	if {[catch {open $file r} fd]} { return }
	if {[array exists ours]} { unset ours }
	foreach line [split [read $fd] \n] {
		if {[regexp {^%!%(.*)!!!(.*)$} $line match name data]} { set ours($name) $data }
	}
	close $fd
}
proc iexbeta_savearray {array file} {
	upvar $array ours
	if {![array exists ours] || [catch {open $file w} fd]} { return }
	foreach entry [array names ours] { puts $fd "%!%$entry!!!$ours($entry)" }
	close $fd
}
proc iexbeta_html2text {text} {
	regsub -all -- {<a (?![^>]*href)[^>]*>} $text {}  text
	regsub -all -- {<[^a][^>]*>} $text {}  text
	regsub -all -- {&nbsp;} $text { } text
	regsub -all -- {&lt;}   $text {<} text
	regsub -all -- {&gt;}   $text {>} text
	regsub -all -- {&quot;} $text {"} text
	regsub -all -- {&#39;}  $text {'} text
	regsub -all -- {&#91;}  $text {[} text
	regsub -all -- {&#93;}  $text {]} text
	regsub -all -- {&copy;} $text {} text
	regsub -all -- {&amp;}  $text {&} text
	return $text
}
proc iexbeta_dopre {} {
	if {!$::iexbeta_setting(IEXBETA:)} { return "" }
	if {!$::iexbeta_setting(bold)} { return "IEXBETA: " }
	return "\002IEXBETA:\002 "
}
proc iexbeta_hbold {text} { return [expr {($::iexbeta_setting(hbold)) ? "\002$text\002" : "$text"}] }
proc iexbeta_notify {text} {
	foreach chan [channels] {
		if {[botonchan $chan] && [lsearch -exact [channel info $chan] "+iexbeta"] != -1} { puthelp "PRIVMSG $chan :$text" }
	}
}
proc iexbeta_notnumber {string} { return [regexp {[^0-9]} $string] }
proc iexbeta_sendtext {nick chan text} {
	switch -- $::iexbeta_setting(rplmode) {
		"0" { puthelp "PRIVMSG $chan :$text" }
		"1" { puthelp "NOTICE $chan :$text" }
		"2" { puthelp "PRIVMSG $nick :$text" }
		"3" { puthelp "NOTICE $nick :$text" }
	}
}

if {![array exists iexbeta_master] && [file exists $iexbeta_setting(database)]} { iexbeta_readarray iexbeta_master $iexbeta_setting(database) }
if {[lsearch -glob [timers] "* iexbeta_update *"] == -1} { timer $iexbeta_setting(updatetime) iexbeta_update }

proc iexbeta_dcc {hand idx text} {
	if {![info exists ::iexbeta_master(data,0)]} { putdcc $idx "[iexbeta_dopre]The news data has not yet been updated." ; return }
	if {[string compare [set rmax [join [lrange [split $text] 0 0]]] ""] == [set ctr 0] || [iexbeta_notnumber [join [lrange [split $text] 0 0]]]} { set rmax $::iexbeta_setting(rmax) }
	while {$ctr < $rmax} {
		if {![info exists ::iexbeta_master(data,$ctr)]} { return }
		putdcc $idx "[iexbeta_dopre][iexbeta_hbold "\[[expr $ctr + 1]\] $::iexbeta_master(headline,$ctr)"]"
		putdcc $idx "[iexbeta_dopre]$::iexbeta_master(data,$ctr)"
		incr ctr
	}
}

proc iexbeta_pub {nick uhost hand chan text} {
	if {![info exists ::iexbeta_master(data,0)]} { iexbeta_sendtext $nick $chan "The news data has not yet been updated." ; return }
	if {[string compare [set rmax [join [lrange [split $text] 0 0]]] ""] == [set ctr 0] || [iexbeta_notnumber [join [lrange [split $text] 0 0]]]} { set rmax $::iexbeta_setting(rmax) }
	while {$ctr < $rmax} {
		if {![info exists ::iexbeta_master(data,$ctr)]} { return }
		iexbeta_sendtext $nick $chan "[iexbeta_hbold "\[[expr $ctr + 1]\] $::iexbeta_master(headline,$ctr)"]"
		iexbeta_sendtext $nick $chan $::iexbeta_master(data,$ctr)
		incr ctr
	}
}

proc iexbeta_update {} {
	global iexbeta_master
	if {[lsearch -glob [timers] "* iexbeta_update *"] == -1} { timer $::iexbeta_setting(updatetime) iexbeta_update }
	if {[catch {set tok [::http::geturl $::iexbeta_setting(url)]} error]} { putlog "[iexbeta_dopre]An error occured while connecting to $::iexbeta_setting(url)." ; return }
	if {[::http::ncode $tok] != 200} {
		putlog "[iexbeta_dopre]An error occured while connecting to $::iexbeta_setting(url)."
		::http::cleanup $tok ; return
	}
	set data [::http::data $tok]
	::http::cleanup $tok
	if {[regexp {<title>Invision Board Database Error</title>} $data]} { putlog "[iexbeta_dopre]An error occured while connecting to $::iexbeta_setting(url)." ; return }
	regsub -all -- {( +)\n} [iexbeta_html2text [string range $data [string first "<table cellpadding=\"4\" cellspacing=\"1\" width=\"100%\" bgcolor=\"#000000\">" $data] [expr [string last "</table><BR></div>" $data] - 1]]] "\n" data
	regsub -all -- {\n[\n ]+} $data "\n" data
	regsub -all -- {^.*?<} $data "<" data
	foreach {a b} [list ctr -1 read 0] { set $a $b }
	foreach {a b c d e} [split [string trim $data] "\n"] {
		incr ctr
		regexp {<a href=['"]([^>]*?)['"]>(.*)$} $a match url headline
		regexp {>(.*)} $b match by1
		regexp {at (.*)} $c match by2
		set temp($ctr) "$url"
		set iexbeta_master(headline,$ctr) "\[$headline ($by1 @ $by2)\]"
	}
	if {![info exists iexbeta_master(data,0)] || [string compare $iexbeta_master(data,0) $temp(0)] != 0} { set read 1 }
	foreach a [array names temp] { set iexbeta_master(data,$a) $temp($a) }
	iexbeta_savearray iexbeta_master $::iexbeta_setting(database)
	if {$read} { iexbeta_announce_new }
}

proc iexbeta_announce_new {} {
	iexbeta_notify "[iexbeta_hbold "$::iexbeta_master(headline,0)"]"
	iexbeta_notify $::iexbeta_master(data,0)
}

putlog "\002IEXBETA:\002 ieXbeta.tcl Version 1.0 by Wcc is loaded."
